import keyboard


class Input:
    pressed_keys = dict()

    @classmethod
    def get_key(cls, key):
        if keyboard.is_pressed(key):
            cls.pressed_keys[key] = True
            return True
        else:
            cls.pressed_keys[key] = False
            return False

    @classmethod
    def get_key_down(cls, key):
        if not keyboard.is_pressed(key):
            cls.pressed_keys[key] = False
            return False
        elif not cls.pressed_keys.get(key):
            cls.pressed_keys[key] = True
            return True
        else:
            return False



from setuptools import setup, find_packages
from os import path
import os


def get_readme():
    here = path.dirname(__file__)
    with open(path.join(here, 'README.md'), encoding='UTF8') as readme_file:
        readme = readme_file.read()
        return readme


def get_history():
    here = path.dirname(__file__)
    with open(path.join(here, 'HISTORY.md'),
              encoding='utf8') as history_file:
        history = history_file.read()
        return history


def get_requirements():
    here = path.dirname(__file__)
    with open(path.join(here, 'requirements.txt'), encoding='UTF8') as req_file:
        req = req_file.read().splitlines()
        if os.name == 'nt':
            req.append('windows-curses')
        return req


setup(
    name="pyplayscii",
    version="0.2.0",
    author="SeBeom Lee",
    description="Object oriented ascii art python game engine",
    long_description=get_readme(),
    install_requires=get_requirements(),
    long_description_content_type="text/markdown",
    url="https://github.com/k2sebeom/pyplayscii",
    packages=find_packages(include=['playscii', 'playscii.games']),
    test_suite="tests",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)



import curses
from _curses import error as size_error
from abc import ABC, abstractmethod
import time


class GameManager(ABC):
    def __init__(self, screen_size):
        self.__game_objects = []
        self.width = screen_size[0]
        self.height = screen_size[1]
        self.__board = []
        self.__clear_board()
        self.__stdscr = None
        self.__flags = dict()
        self.__flags['quit'] = False
        self.delta_time = 0
        self.__old_time = time.time()
        self.__title = ""

    def find(self, obj_class):
        obj_with_class = []
        for game_object in self.__game_objects:
            if isinstance(game_object, obj_class):
                obj_with_class.append(game_object)
        return obj_with_class

    def add_object(self, obj):
        self.__game_objects.append(obj)

    def set_title(self, title):
        self.__title = title

    def start(self):
        self.__stdscr = curses.initscr()
        self.setup()
        while True:
            if self.__flags['quit']:
                break
            curr_time = time.time()
            self.delta_time = curr_time - self.__old_time
            self.__old_time = curr_time
            self.update()
            self.__clear_board()
            for game_object in self.__game_objects:
                game_object.delta_time = self.delta_time
                game_object.update()
                game_object.draw(self.__board)
            try:
                self.__update_board()
            except size_error:
                curses.resize_term(self.height + 5, self.width + 10)
                self.__stdscr.refresh()
            time.sleep(0.02)

    def set_flag(self, flag_key, flag_value):
        if flag_key not in self.__flags:
            raise KeyError
        else:
            self.__flags[flag_key] = flag_value

    def quit(self):
        self.set_flag('quit', True)

    def __clear_board(self):
        self.__board = [[" " for _ in range(self.width)]
                        for _ in range(self.height)]

    def __update_board(self):
        self.__stdscr.clear()
        padding = ' ' * ((curses.COLS - self.width) // 2)
        self.__stdscr.addstr(padding + '-' * (self.width + 2) + '\n')
        self.__stdscr.addstr(padding + f"|{self.__title:^{self.width}}|\n")
        self.__stdscr.addstr(padding + '-' * (self.width + 2) + '\n')
        for row in self.__board:
            self.__stdscr.addstr(padding + '|')
            for curr in row:
                self.__stdscr.addstr(f"{curr:^1}")
            self.__stdscr.addstr('|\n')
        self.__stdscr.addstr(padding + '-' * (self.width + 2))
        self.__stdscr.refresh()

    def get_flag(self, flag_key):
        if flag_key not in self.__flags:
            raise KeyError
        else:
            return self.__flags[flag_key]

    @property
    def game_objects(self):
        return self.__game_objects

    @abstractmethod
    def setup(self):
        pass

    @abstractmethod
    def update(self):
        pass


class GameObject(ABC):
    def __init__(self, pos=(0, 0), render='', size=(0, 0)):
        self.x = pos[0]
        self.y = pos[1]
        self.render = render
        self.delta_time = 0
        self.width = size[0]
        self.height = size[1]
        self.__parent = None

    @property
    def parent(self):
        return self.__parent

    def draw(self, board):
        render_text = self.render.split('\n')
        x, y = int(self.x), int(self.y)
        if len(render_text) == 1 and render_text[0] == '':
            return
        for i in range(len(render_text)):
            if not (0 <= y - i < len(board)):
                break
            for j in range(len(render_text[i])):
                if not (0 <= x + j < len(board[0])):
                    continue
                board[~(y - i)][x + j] = render_text[i][j]

    def on_collision(self, other):
        return (
                       other.x - other.width <= self.x
                       <= other.x + other.width
               ) and (
                       other.y - other.height <= self.y
                       <= other.y + other.height
               )

    def update(self):
        pass



from playscii import GameManager, GameObject
from threading import Timer
from random import randint

JET_RENDER = "  /\\\n" \
             "  []\n" \
             "< () >\n" \
             "  <>"

ENEMY_RENDER = " __\n" \
               "<oo>"

WIDTH, HEIGHT, SPEED, ENEMY_SPEED = 50, 25, 30, 5


class GalagaManager(GameManager):
    def __init__(self, gyro, button, led):
        super().__init__((WIDTH, HEIGHT))
        self.gyro = gyro
        self.button = button
        self.led = led
        self.time = 0
        self.jet = Jet(pos=(WIDTH // 2, 3))
        self.cooling = False
        self.enemies = []
        self.bullets = []
        self.spawn_time = 3
        self.score = 0
        self.scoreboard = GameObject((WIDTH // 2, HEIGHT - 2), str(self.score))

    def setup(self):
        self.time = 0
        self.add_object(self.jet)
        self.set_title("Galaga with PyMODI")
        self.led.set_green()
        self.add_object(self.scoreboard)

    def update(self):
        self.time += self.delta_time

        if self.time > self.spawn_time:
            self.time = 0
            self.spawn_enemy()
        self.scoreboard.render = str(self.score)
        pitch = self.gyro.get_pitch()
        if pitch < 5 and self.jet.x < self.width - self.jet.width:
            self.jet.x += SPEED * self.delta_time
        elif pitch > 5 and self.jet.x > 0:
            self.jet.x -= SPEED * self.delta_time
        if self.button.get_pressed():
            if not self.cooling:
                self.cooling = True
                self.jet.shoot(self)
                Timer(0.2, self.cool_off).start()
        if self.jet.check_death(self.enemies):
            self.set_title("GAME OVER")
            self.led.set_rgb(255, 0, 0)
            self.quit()
            return
        self.clean_enemies()
        self.clean_bullets()

    def cool_off(self):
        self.cooling = False

    def spawn_enemy(self):
        enemy = Enemy((randint(0, WIDTH - 6), HEIGHT))
        self.add_object(enemy)
        self.enemies.append(enemy)

    def clean_enemies(self):
        i = 0
        while i < len(self.enemies):
            enemy = self.enemies[i]
            if enemy.dead or enemy.check_death(self.bullets):
                self.enemies.pop(i)
                self.score += 1
                if self.spawn_time > 0.7:
                    self.spawn_time -= 0.1
                self.game_objects.remove(enemy)
            else:
                i -= -1

    def clean_bullets(self):
        i = 0
        while i < len(self.bullets):
            bullet = self.bullets[i]
            if bullet.hit:
                self.bullets.pop(i)
                self.game_objects.remove(bullet)
            else:
                i -= -1


class Bullet(GameObject):
    def __init__(self, pos, render):
        super().__init__(pos, render, (2, 2))
        self.hit = False

    def update(self):
        if self.y < HEIGHT:
            self.y += 20 * self.delta_time
        else:
            self.hit = True


class Jet(GameObject):
    def __init__(self, pos):
        super().__init__(pos, JET_RENDER, (4, 2))

    def update(self):
        pass

    def shoot(self, manager):
        bullet = Bullet((self.x, self.y), '   |')
        manager.add_object(bullet)
        manager.bullets.append(bullet)

    def check_death(self, enemies):
        for enemy in enemies:
            if self.on_collision(enemy):
                return True
        return False


class Enemy(GameObject):
    def __init__(self, pos):
        super().__init__(pos, ENEMY_RENDER, (4, 2))
        self.dead = False

    def update(self):
        if self.y >= -5:
            self.y -= ENEMY_SPEED * self.delta_time
        else:
            self.dead = True

    def check_death(self, bullets):
        for bullet in bullets:
            if bullet.on_collision(self):
                bullet.hit = True
                return True
        return False
